@@ -43,6 +43,7 @@ import (
 
 var _ = Describe("NamespacedClient", func() {
 	var dep *appsv1.Deployment
+	var nameSpace *corev1.Namespace
 	var acDep *appsv1applyconfigurations.DeploymentApplyConfiguration
 	var ns = "default"
 	var count uint64 = 0
@@ -53,6 +54,8 @@ var _ = Describe("NamespacedClient", func() {
 
 		err := rbacv1.AddToScheme(sch)
 		Expect(err).ToNot(HaveOccurred())
+		err = corev1.AddToScheme(sch)
+		Expect(err).ToNot(HaveOccurred())
 		err = appsv1.AddToScheme(sch)
 		Expect(err).ToNot(HaveOccurred())
 
@@ -80,6 +83,12 @@ var _ = Describe("NamespacedClient", func() {
 				},
 			},
 		}
+		nameSpace = &corev1.Namespace{
+			ObjectMeta: metav1.ObjectMeta{
+				Name:   fmt.Sprintf("namespace-%v", count),
+				Labels: map[string]string{"name": fmt.Sprintf("namespace-%v", count)},
+			},
+		}
 		acDep = appsv1applyconfigurations.Deployment(dep.Name, "").
 			WithLabels(dep.Labels).
 			WithSpec(appsv1applyconfigurations.DeploymentSpec().
@@ -131,10 +140,13 @@ var _ = Describe("NamespacedClient", func() {
 			var err error
 			dep, err = clientset.AppsV1().Deployments(ns).Create(ctx, dep, metav1.CreateOptions{})
 			Expect(err).NotTo(HaveOccurred())
+			nameSpace, err = clientset.CoreV1().Namespaces().Create(ctx, nameSpace, metav1.CreateOptions{})
+			Expect(err).NotTo(HaveOccurred())
 		})
 
 		AfterEach(func(ctx SpecContext) {
 			deleteDeployment(ctx, dep, ns)
+			deleteNamespace(ctx, nameSpace)
 		})
 
 		It("should successfully List objects when namespace is not specified with the object", func(ctx SpecContext) {
@@ -146,6 +158,13 @@ var _ = Describe("NamespacedClient", func() {
 			Expect(result.Items[0]).To(BeEquivalentTo(*dep))
 		})
 
+		It("should successfully List objects when object is not namespaced scoped", func(ctx SpecContext) {
+			result := &corev1.NamespaceList{}
+			opts := &client.ListOptions{}
+			Expect(getClient().List(ctx, result, opts)).NotTo(HaveOccurred())
+			Expect(result.Items).NotTo(BeEmpty())
+		})
+
 		It("should List objects from the namespace specified in the client", func(ctx SpecContext) {
 			result := &appsv1.DeploymentList{}
 			opts := client.InNamespace("non-default")
