@@ -20,11 +20,20 @@ package v1beta1
 const (
 	// MetaRevisionKey is the key used to hold the source artifact revision.
 	MetaRevisionKey string = "revision"
+	// MetaOriginRevisionKey is the key used to hold the source artifact origin revision.
+	MetaOriginRevisionKey string = "originRevision"
 	// MetaChecksumKey is the key used to hold the source artifact checksum.
 	// Deprecated: in favor of MetaDigestKey.
 	MetaChecksumKey string = "checksum"
 	// MetaDigestKey is the key used to hold the source artifact digest.
 	MetaDigestKey string = "digest"
+	// MetaTokenKey is the key used to hold an arbitrary token whose contents
+	// are defined on a per-event-emitter basis for uniquely identifying the
+	// contents of the event payload. For example, it could be the generation
+	// of an object, or the hash of a set of configurations, or even a
+	// base64-encoded set of configurations. This is useful for example for
+	// rate limiting the events.
+	MetaTokenKey string = "token"
 	// MetaCommitStatusKey is the key used to signal a Git commit status event.
 	MetaCommitStatusKey string = "commit_status"
 	// MetaCommitStatusUpdateValue is the value of MetaCommitStatusKey
